library neuli.ui;

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nueli_core/nueli_core.dart';

const ui = UITools();

abstract class UI extends StatefulWidget {
  final Ref<BuildContext> baseContext = $();
  final Ref<Function(Function() fn)> baseSetState = $((fn) => fn());

  UI({Key? key}) : super(key: key);

  @mustCallSuper
  Decor get empty => ui.empty;

  @mustCallSuper
  Decor gap(double size) => ui.gap(size);

  @mustCallSuper
  Decor dot(double size, [Color? color]) => ui.dot(size, color ?? palette.onBackground);

  @mustCallSuper
  Decor lineH(double thickness, [Color? color]) => ui.lineH(thickness, color ?? palette.onBackground);

  @mustCallSuper
  Decor lineV(double thickness, [Color? color]) => ui.lineV(thickness, color ?? palette.onBackground);

  @mustCallSuper
  Expanded flex(int flex) => ui.flex(flex);

  BuildContext get context => baseContext.get!;

  void setState(Function() fn) => baseSetState.get!(fn);

  @mustCallSuper
  Widget call();

  @override
  UIState createState() => UIState();
}

class UIState<T extends UI> extends State<T> {
  @override
  @mustCallSuper
  Widget build(BuildContext context) {
    widget.baseContext.set(context);
    widget.baseSetState.set(setState);
    return widget();
  }
}

class UITools {
  const UITools();

  Decor get empty => $decor;

  Decor gap(double size) {
    return $decor.size(size);
  }

  Decor lineH(double thickness, [Color? color]) {
    return $decor
      ..widthToInfinity.height(thickness)
      ..bucketFill(color ?? palette.accent)
      ..copyRadii(Radii.all(thickness));
  }

  Decor lineV(double thickness, [Color? color]) {
    return $decor
      ..width(thickness).heightToInfinity
      ..bucketFill(color ?? palette.accent)
      ..copyRadii(Radii.all(thickness));
  }

  Decor dot(double size, [Color? color]) {
    return $decor
      ..size(size)
      ..bucketFill(color ?? palette.accent)
      ..copyRadii(Radii.all(size));
  }

  Expanded flex(int flex) => Expanded(child: empty, flex: flex);
}

extension ExpandedUI on Expanded {
  Expanded operator &(Decor child) => Expanded(child: child, key: key, flex: flex);
}
